@@ -5,13 +5,12 @@ from .conn_msgp import send_packet
 from .pair import SocketChannel
 from .util import WaitGroup, pickle_dumps, pickle_loads, timeblock
 
-import os
 import secrets
-import sys
 from functools import partial
 from math import inf as INFIN
 import getpass
 import argparse
+from pathlib import Path
 import logging
 logger = logging.getLogger(__name__)
 
@@ -51,7 +50,7 @@ class RemoteExecutor:
             async for tid, res, fargs in self.pull_job:
                 self._wg.add()
                 await send_packet(self._c._so, dict(tid=tid, res=res, func=pickle_dumps(fargs))) # We don't need locking
-    async def run(self):
+    async def run(self, task_status=trio.TASK_STATUS_IGNORED):
         if self.gnaw is DAEMON: # gnaw daemon with unixconn
             # process detection
             whoami = getpass.getuser()
@@ -74,17 +73,25 @@ class RemoteExecutor:
                 logger.info(f"starting a Gnaw instance at {self.listen!r}")
                 self.gnaw = f"unix:///tmp/gnaw-{secrets.token_urlsafe()}"
                 conf = self.gnaw_conf
-                _ = await trio.open_process(
+                gnawproc = await trio.lowlevel.open_process(
                     ["gnaw", "-hurl", self.gnaw, "-wurl", self.listen, "-n", str(conf.max_conn),
                              "-log", conf.log_file, "-t", conf.idle_quit, "-swarm", str(conf.swarm), *conf.extra_args],
                     start_new_session=True, # daemon
                 )
-                await trio.sleep(0.1) # wait for gnaw startup
+                for retry in range(7): # wait for gnaw startup
+                    await trio.sleep(0.1 * 2**retry)
+                    if Path(self.gnaw[len('unix://'):]).exists():
+                        break
+                    if gnawproc.returncode is not None:
+                        raise RuntimeError(f"failed to start Gnaw, exit code {gnawproc.returncode}")
+                else:
+                    raise RuntimeError("Gnaw took too long to start")
         with timeblock("all jobs"):
             async with SocketChannel(self.gnaw, dial=True, _n=self._n) as self._c, \
                        self.push_result:
                 await send_packet(self._c._so, dict(name=self.name)) # handshake
                 self._n.start_soon(self._sender)
+                task_status.started()
                 async for x in self._c:
                     if x['exception']: # display exception
                         tb, exp = pickle_loads(x['result'])
@@ -95,7 +102,7 @@ class RemoteExecutor:
             if not self._c.is_clean:
                 raise RuntimeError("connection to Gnaw lost unexpectedly")
     async def __aenter__(self):
-        self._n.start_soon(self.run)
+        await self._n.start(self.run)
         return self
     async def __aexit__(self, *exc):
         if any(exc):
